==> conf_ini_g-2024.9.tar.gz: conf_ini_g-2024.9/package/conf_ini_g/phase/typed/section/units.py <==
"""
Copyright CNRS/Inria/UniCA
Contributor(s): Eric Debreuve (eric.debreuve@cnrs.fr) since 2021
SEE COPYRIGHT NOTICE BELOW
"""

from conf_ini_g.interface.storage.parameter import INI_COMMENT_MARKER
from conf_ini_g.interface.storage.section import INI_UNIT_SECTION
from conf_ini_g.phase.specification.config.main import config_t as specification_t
from conf_ini_g.phase.untyped.parameter import ValueUnitAndComment
from conf_ini_g.phase.untyped.section import STD_UNIT_CONVERSIONS


def InitialUnits(specification: specification_t, /) -> dict[str, float]:
    """
    Specification can override standard units.
    """
    units = dict(STD_UNIT_CONVERSIONS)
    if INI_UNIT_SECTION in specification:
        units.update(specification[INI_UNIT_SECTION].AsDict())

    return units


def UpdateUnits(
    units: dict[str, float], name: str, value: str, /
) -> tuple[float | None, list[str]]:
    """"""
    issues = []

    if name in STD_UNIT_CONVERSIONS:
        return None, [f'Standard unit "{name}" cannot be overriden.']

    value, *_ = ValueUnitAndComment(value, INI_COMMENT_MARKER)
    try:
        value = float(value)
    except ValueError:
        return None, [f"/{INI_UNIT_SECTION}.{name}/ Invalid value: {value}."]

    units[name] = value

    return value, issues


"""
COPYRIGHT NOTICE

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

SEE LICENCE NOTICE: file README-LICENCE-utf8.txt at project source root.

This software is being developed by Eric Debreuve, a CNRS employee and
member of team Morpheme.
Team Morpheme is a joint team between Inria, CNRS, and UniCA.
It is hosted by the Centre Inria d'Université Côte d'Azur, Laboratory
I3S, and Laboratory iBV.

CNRS: https://www.cnrs.fr/index.php/en
Inria: https://www.inria.fr/en/
UniCA: https://univ-cotedazur.eu/
Centre Inria d'Université Côte d'Azur: https://www.inria.fr/en/centre/sophia/
I3S: https://www.i3s.unice.fr/en/
iBV: http://ibv.unice.fr/
Team Morpheme: https://team.inria.fr/morpheme/
"""

==> conf_ini_g-2024.13.tar.gz: conf_ini_g-2024.13/package/conf_ini_g/phase/typed/section/units.py <==
"""
Copyright CNRS/Inria/UniCA
Contributor(s): Eric Debreuve (eric.debreuve@cnrs.fr) since 2021
SEE COPYRIGHT NOTICE BELOW
"""

from conf_ini_g.interface.storage.parameter import INI_COMMENT_MARKER
from conf_ini_g.interface.storage.section import INI_UNIT_SECTION
from conf_ini_g.phase.definition.config.main import config_t as specification_t
from conf_ini_g.phase.untyped.parameter import ValueUnitAndComment
from conf_ini_g.phase.untyped.section import STD_UNIT_CONVERSIONS


def InitialUnits(specification: specification_t, /) -> dict[str, float]:
    """
    Specification can override standard units.
    """
    units = dict(STD_UNIT_CONVERSIONS)
    if INI_UNIT_SECTION in specification:
        units.update(specification[INI_UNIT_SECTION].AsDict())

    return units


def UpdateUnits(
    units: dict[str, float], name: str, value: str, /
) -> tuple[float | None, list[str]]:
    """"""
    issues = []

    if name in STD_UNIT_CONVERSIONS:
        return None, [f'Standard unit "{name}" cannot be overriden.']

    value, *_ = ValueUnitAndComment(value, INI_COMMENT_MARKER)
    try:
        value = float(value)
    except ValueError:
        return None, [f"/{INI_UNIT_SECTION}.{name}/ Invalid value: {value}."]

    units[name] = value

    return value, issues


"""
COPYRIGHT NOTICE

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

SEE LICENCE NOTICE: file README-LICENCE-utf8.txt at project source root.

This software is being developed by Eric Debreuve, a CNRS employee and
member of team Morpheme.
Team Morpheme is a joint team between Inria, CNRS, and UniCA.
It is hosted by the Centre Inria d'Université Côte d'Azur, Laboratory
I3S, and Laboratory iBV.

CNRS: https://www.cnrs.fr/index.php/en
Inria: https://www.inria.fr/en/
UniCA: https://univ-cotedazur.eu/
Centre Inria d'Université Côte d'Azur: https://www.inria.fr/en/centre/sophia/
I3S: https://www.i3s.unice.fr/en/
iBV: http://ibv.unice.fr/
Team Morpheme: https://team.inria.fr/morpheme/
"""
